@@ -151,11 +151,8 @@ impl<'a> MutationVisitor<'a> {
     }
 }
 
-impl<'a, 'b> StatementVisitor<'b> for MutationVisitor<'a>
-where
-    'b: 'a,
-{
-    fn visit_stmt(&mut self, stmt: &'b Stmt) {
+impl<'a> StatementVisitor<'a> for MutationVisitor<'a> {
+    fn visit_stmt(&mut self, stmt: &'a Stmt) {
         if match_mutation(stmt, self.target) {
             self.is_mutated = true;
         } else {
