@@ -8,6 +8,7 @@ use ruff_diagnostics::Diagnostic;
 use ruff_diagnostics::Edit;
 use ruff_diagnostics::Fix;
 use ruff_macros::{derive_message_formats, violation};
+use ruff_python_ast::PySourceType;
 use ruff_python_codegen::Stylist;
 use ruff_python_parser::lexer::LexResult;
 use ruff_python_parser::lexer::LexicalError;
@@ -24,7 +25,7 @@ use ruff_python_trivia::PythonWhitespace;
 /// Number of blank lines around top level classes and functions.
 const BLANK_LINES_TOP_LEVEL: u32 = 2;
 /// Number of blank lines around methods and nested classes and functions.
-const BLANK_LINES_METHOD_LEVEL: u32 = 1;
+const BLANK_LINES_NESTED_LEVEL: u32 = 1;
 
 /// ## What it does
 /// Checks for missing blank lines between methods of a class.
@@ -51,16 +52,21 @@ const BLANK_LINES_METHOD_LEVEL: u32 = 1;
 ///         pass
 /// ```
 ///
+/// ## Typing stub files (`.pyi`)
+/// The typing style guide recommends to not use blank lines between methods except to group
+/// them. That's why this rule is not enabled in typing stub files.
+///
 /// ## References
 /// - [PEP 8](https://peps.python.org/pep-0008/#blank-lines)
 /// - [Flake 8 rule](https://www.flake8rules.com/rules/E301.html)
+/// - [Typing Style Guide](https://typing.readthedocs.io/en/latest/source/stubs.html#blank-lines)
 #[violation]
 pub struct BlankLineBetweenMethods;
 
 impl AlwaysFixableViolation for BlankLineBetweenMethods {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Expected {BLANK_LINES_METHOD_LEVEL:?} blank line, found 0")
+        format!("Expected {BLANK_LINES_NESTED_LEVEL:?} blank line, found 0")
     }
 
     fn fix_title(&self) -> String {
@@ -74,6 +80,10 @@ impl AlwaysFixableViolation for BlankLineBetweenMethods {
 /// ## Why is this bad?
 /// PEP 8 recommends exactly two blank lines between top level functions and classes.
 ///
+/// Note: The rule respects the [`lint.isort.lines-after-imports`] setting when determining
+/// the required number of blank lines between top-level `import` statements and function or class definitions
+/// for compatibility with isort.
+///
 /// ## Example
 /// ```python
 /// def func1():
@@ -92,22 +102,29 @@ impl AlwaysFixableViolation for BlankLineBetweenMethods {
 ///     pass
 /// ```
 ///
+/// ## Typing stub files (`.pyi`)
+/// The typing style guide recommends to not use blank lines between classes and functions except to group
+/// them. That's why this rule is not enabled in typing stub files.
+///
 /// ## References
 /// - [PEP 8](https://peps.python.org/pep-0008/#blank-lines)
 /// - [Flake 8 rule](https://www.flake8rules.com/rules/E302.html)
+/// - [Typing Style Guide](https://typing.readthedocs.io/en/latest/source/stubs.html#blank-lines)
 #[violation]
 pub struct BlankLinesTopLevel {
     actual_blank_lines: u32,
+    expected_blank_lines: u32,
 }
 
 impl AlwaysFixableViolation for BlankLinesTopLevel {
     #[derive_message_formats]
     fn message(&self) -> String {
         let BlankLinesTopLevel {
-            actual_blank_lines: nb_blank_lines,
+            actual_blank_lines,
+            expected_blank_lines,
         } = self;
 
-        format!("Expected {BLANK_LINES_TOP_LEVEL:?} blank lines, found {nb_blank_lines}")
+        format!("Expected {expected_blank_lines:?} blank lines, found {actual_blank_lines}")
     }
 
     fn fix_title(&self) -> String {
@@ -144,9 +161,17 @@ impl AlwaysFixableViolation for BlankLinesTopLevel {
 ///     pass
 /// ```
 ///
+/// ## Typing stub files (`.pyi`)
+/// The rule allows at most one blank line in typing stub files in accordance to the typing style guide recommendation.
+///
+/// Note: The rule respects the following `isort` settings when determining the maximum number of blank lines allowed between two statements:
+/// * [`lint.isort.lines-after-imports`]: For top-level statements directly following an import statement.
+/// * [`lint.isort.lines-between-types`]: For `import` statements directly following a `from ... import ...` statement or vice versa.
+///
 /// ## References
 /// - [PEP 8](https://peps.python.org/pep-0008/#blank-lines)
 /// - [Flake 8 rule](https://www.flake8rules.com/rules/E303.html)
+/// - [Typing Style Guide](https://typing.readthedocs.io/en/latest/source/stubs.html#blank-lines)
 #[violation]
 pub struct TooManyBlankLines {
     actual_blank_lines: u32,
@@ -155,10 +180,9 @@ pub struct TooManyBlankLines {
 impl AlwaysFixableViolation for TooManyBlankLines {
     #[derive_message_formats]
     fn message(&self) -> String {
-        let TooManyBlankLines {
-            actual_blank_lines: nb_blank_lines,
-        } = self;
-        format!("Too many blank lines ({nb_blank_lines})")
+        let TooManyBlankLines { actual_blank_lines } = self;
+
+        format!("Too many blank lines ({actual_blank_lines})")
     }
 
     fn fix_title(&self) -> String {
@@ -217,7 +241,7 @@ impl AlwaysFixableViolation for BlankLineAfterDecorator {
 /// Checks for missing blank lines after the end of function or class.
 ///
 /// ## Why is this bad?
-/// PEP 8 recommends using blank lines as following:
+/// PEP 8 recommends using blank lines as follows:
 /// - Two blank lines are expected between functions and classes
 /// - One blank line is expected between methods of a class.
 ///
@@ -237,9 +261,14 @@ impl AlwaysFixableViolation for BlankLineAfterDecorator {
 /// user = User()
 /// ```
 ///
+/// ## Typing stub files (`.pyi`)
+/// The typing style guide recommends to not use blank lines between statements except to group
+/// them. That's why this rule is not enabled in typing stub files.
+///
 /// ## References
 /// - [PEP 8](https://peps.python.org/pep-0008/#blank-lines)
 /// - [Flake 8 rule](https://www.flake8rules.com/rules/E305.html)
+/// - [Typing Style Guide](https://typing.readthedocs.io/en/latest/source/stubs.html#blank-lines)
 #[violation]
 pub struct BlankLinesAfterFunctionOrClass {
     actual_blank_lines: u32,
@@ -263,7 +292,7 @@ impl AlwaysFixableViolation for BlankLinesAfterFunctionOrClass {
 /// Checks for 1 blank line between nested function or class definitions.
 ///
 /// ## Why is this bad?
-/// PEP 8 recommends using blank lines as following:
+/// PEP 8 recommends using blank lines as follows:
 /// - Two blank lines are expected between functions and classes
 /// - One blank line is expected between methods of a class.
 ///
@@ -286,9 +315,14 @@ impl AlwaysFixableViolation for BlankLinesAfterFunctionOrClass {
 ///         pass
 /// ```
 ///
+/// ## Typing stub files (`.pyi`)
+/// The typing style guide recommends to not use blank lines between classes and functions except to group
+/// them. That's why this rule is not enabled in typing stub files.
+///
 /// ## References
 /// - [PEP 8](https://peps.python.org/pep-0008/#blank-lines)
 /// - [Flake 8 rule](https://www.flake8rules.com/rules/E306.html)
+/// - [Typing Style Guide](https://typing.readthedocs.io/en/latest/source/stubs.html#blank-lines)
 #[violation]
 pub struct BlankLinesBeforeNestedDefinition;
 
@@ -415,6 +449,8 @@ impl<'a> Iterator for LinePreprocessor<'a> {
                     {
                         LogicalLineKind::Function
                     }
+                    TokenKind::Import => LogicalLineKind::Import,
+                    TokenKind::From => LogicalLineKind::FromImport,
                     _ => LogicalLineKind::Other,
                 };
 
@@ -560,9 +596,17 @@ enum Follows {
     Other,
     Decorator,
     Def,
+    Import,
+    FromImport,
     Docstring,
 }
 
+impl Follows {
+    const fn is_any_import(self) -> bool {
+        matches!(self, Follows::Import | Follows::FromImport)
+    }
+}
+
 #[derive(Copy, Clone, Debug, Default)]
 enum Status {
     /// Stores the indent level where the nesting started.
@@ -602,8 +646,351 @@ impl Status {
 }
 
 /// Contains variables used for the linting of blank lines.
-#[derive(Debug, Default)]
-pub(crate) struct BlankLinesChecker {
+#[derive(Debug)]
+pub(crate) struct BlankLinesChecker<'a> {
+    stylist: &'a Stylist<'a>,
+    locator: &'a Locator<'a>,
+    indent_width: IndentWidth,
+    lines_after_imports: isize,
+    lines_between_types: usize,
+    source_type: PySourceType,
+}
+
+impl<'a> BlankLinesChecker<'a> {
+    pub(crate) fn new(
+        locator: &'a Locator<'a>,
+        stylist: &'a Stylist<'a>,
+        settings: &crate::settings::LinterSettings,
+        source_type: PySourceType,
+    ) -> BlankLinesChecker<'a> {
+        BlankLinesChecker {
+            stylist,
+            locator,
+            indent_width: settings.tab_size,
+            lines_after_imports: settings.isort.lines_after_imports,
+            lines_between_types: settings.isort.lines_between_types,
+            source_type,
+        }
+    }
+
+    /// E301, E302, E303, E304, E305, E306
+    pub(crate) fn check_lines(&self, tokens: &[LexResult], diagnostics: &mut Vec<Diagnostic>) {
+        let mut prev_indent_length: Option<usize> = None;
+        let mut state = BlankLinesState::default();
+        let line_preprocessor = LinePreprocessor::new(tokens, self.locator, self.indent_width);
+
+        for logical_line in line_preprocessor {
+            // Reset `follows` after a dedent:
+            // ```python
+            // if True:
+            //      import test
+            // a = 10
+            // ```
+            // The `a` statement doesn't follow the `import` statement but the `if` statement.
+            if let Some(prev_indent_length) = prev_indent_length {
+                if prev_indent_length > logical_line.indent_length {
+                    state.follows = Follows::Other;
+                }
+            }
+
+            state.class_status.update(&logical_line);
+            state.fn_status.update(&logical_line);
+
+            self.check_line(&logical_line, &state, prev_indent_length, diagnostics);
+
+            match logical_line.kind {
+                LogicalLineKind::Class => {
+                    if matches!(state.class_status, Status::Outside) {
+                        state.class_status = Status::Inside(logical_line.indent_length);
+                    }
+                    state.follows = Follows::Other;
+                }
+                LogicalLineKind::Decorator => {
+                    state.follows = Follows::Decorator;
+                }
+                LogicalLineKind::Function => {
+                    if matches!(state.fn_status, Status::Outside) {
+                        state.fn_status = Status::Inside(logical_line.indent_length);
+                    }
+                    state.follows = Follows::Def;
+                }
+                LogicalLineKind::Comment => {}
+                LogicalLineKind::Import => {
+                    state.follows = Follows::Import;
+                }
+                LogicalLineKind::FromImport => {
+                    state.follows = Follows::FromImport;
+                }
+                LogicalLineKind::Other => {
+                    state.follows = Follows::Other;
+                }
+            }
+
+            if logical_line.is_docstring {
+                state.follows = Follows::Docstring;
+            }
+
+            if !logical_line.is_comment_only {
+                state.is_not_first_logical_line = true;
+
+                state.last_non_comment_line_end = logical_line.logical_line_end;
+
+                if logical_line.indent_length == 0 {
+                    state.previous_unindented_line_kind = Some(logical_line.kind);
+                }
+            }
+
+            if !logical_line.is_comment_only {
+                prev_indent_length = Some(logical_line.indent_length);
+            }
+        }
+    }
+
+    #[allow(clippy::nonminimal_bool)]
+    fn check_line(
+        &self,
+        line: &LogicalLineInfo,
+        state: &BlankLinesState,
+        prev_indent_length: Option<usize>,
+        diagnostics: &mut Vec<Diagnostic>,
+    ) {
+        if line.preceding_blank_lines == 0
+            // Only applies to methods.
+            && matches!(line.kind,  LogicalLineKind::Function | LogicalLineKind::Decorator)
+            // Allow groups of one-liners.
+            && !(matches!(state.follows, Follows::Def) && !matches!(line.last_token, TokenKind::Colon))
+            && matches!(state.class_status, Status::Inside(_))
+            // The class/parent method's docstring can directly precede the def.
+            // Allow following a decorator (if there is an error it will be triggered on the first decorator).
+            && !matches!(state.follows, Follows::Docstring | Follows::Decorator)
+            // Do not trigger when the def follows an if/while/etc...
+            && prev_indent_length.is_some_and(|prev_indent_length| prev_indent_length >= line.indent_length)
+            // Blank lines in stub files are only used for grouping. Don't enforce blank lines.
+            && !self.source_type.is_stub()
+        {
+            // E301
+            let mut diagnostic = Diagnostic::new(BlankLineBetweenMethods, line.first_token_range);
+            diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
+                self.stylist.line_ending().to_string(),
+                self.locator.line_start(state.last_non_comment_line_end),
+            )));
+
+            diagnostics.push(diagnostic);
+        }
+
+        // Blank lines in stub files are used to group definitions. Don't enforce blank lines.
+        let max_lines_level = if self.source_type.is_stub() {
+            1
+        } else {
+            if line.indent_length == 0 {
+                BLANK_LINES_TOP_LEVEL
+            } else {
+                BLANK_LINES_NESTED_LEVEL
+            }
+        };
+
+        let expected_blank_lines_before_definition = if line.indent_length == 0 {
+            // Mimic the isort rules for the number of blank lines before classes and functions
+            if state.follows.is_any_import() {
+                // Fallback to the default if the value is too large for an u32 or if it is negative.
+                // A negative value means that isort should determine the blank lines automatically.
+                // `isort` defaults to 2 if before a class or function definition (except in stubs where it is one) and 1 otherwise.
+                // Defaulting to 2 (or 1 in stubs) here is correct because the variable is only used when testing the
+                // blank lines before a class or function definition.
+                u32::try_from(self.lines_after_imports).unwrap_or(max_lines_level)
+            } else {
+                max_lines_level
+            }
+        } else {
+            max_lines_level
+        };
+
+        if line.preceding_blank_lines < expected_blank_lines_before_definition
+            // Allow following a decorator (if there is an error it will be triggered on the first decorator).
+            && !matches!(state.follows, Follows::Decorator)
+            // Allow groups of one-liners.
+            && !(matches!(state.follows, Follows::Def) && !matches!(line.last_token, TokenKind::Colon))
+            // Only trigger on non-indented classes and functions (for example functions within an if are ignored)
+            && line.indent_length == 0
+            // Only apply to functions or classes.
+            && line.kind.is_class_function_or_decorator()
+            // Blank lines in stub files are used to group definitions. Don't enforce blank lines.
+            && !self.source_type.is_stub()
+            // Do not expect blank lines before the first logical line.
+            && state.is_not_first_logical_line
+        {
+            // E302
+            let mut diagnostic = Diagnostic::new(
+                BlankLinesTopLevel {
+                    actual_blank_lines: line.preceding_blank_lines.count(),
+                    expected_blank_lines: expected_blank_lines_before_definition,
+                },
+                line.first_token_range,
+            );
+
+            if let Some(blank_lines_range) = line.blank_lines.range() {
+                diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+                    self.stylist
+                        .line_ending()
+                        .repeat(expected_blank_lines_before_definition as usize),
+                    blank_lines_range,
+                )));
+            } else {
+                diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
+                    self.stylist
+                        .line_ending()
+                        .repeat(expected_blank_lines_before_definition as usize),
+                    self.locator.line_start(state.last_non_comment_line_end),
+                )));
+            }
+
+            diagnostics.push(diagnostic);
+        }
+
+        // If between `import` and `from .. import ..` or the other way round,
+        // allow up to `lines_between_types` newlines for isort compatibility.
+        // We let `isort` remove extra blank lines when the imports belong
+        // to different sections.
+        let max_blank_lines = if matches!(
+            (line.kind, state.follows),
+            (LogicalLineKind::Import, Follows::FromImport)
+                | (LogicalLineKind::FromImport, Follows::Import)
+        ) {
+            max_lines_level.max(u32::try_from(self.lines_between_types).unwrap_or(u32::MAX))
+        } else {
+            expected_blank_lines_before_definition
+        };
+
+        if line.blank_lines > max_blank_lines {
+            // E303
+            let mut diagnostic = Diagnostic::new(
+                TooManyBlankLines {
+                    actual_blank_lines: line.blank_lines.count(),
+                },
+                line.first_token_range,
+            );
+
+            if let Some(blank_lines_range) = line.blank_lines.range() {
+                if max_blank_lines == 0 {
+                    diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(blank_lines_range)));
+                } else {
+                    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+                        self.stylist.line_ending().repeat(max_blank_lines as usize),
+                        blank_lines_range,
+                    )));
+                }
+            }
+
+            diagnostics.push(diagnostic);
+        }
+
+        if matches!(state.follows, Follows::Decorator)
+            && !line.is_comment_only
+            && line.preceding_blank_lines > 0
+        {
+            // E304
+            let mut diagnostic = Diagnostic::new(
+                BlankLineAfterDecorator {
+                    actual_blank_lines: line.preceding_blank_lines.count(),
+                },
+                line.first_token_range,
+            );
+
+            // Get all the lines between the last decorator line (included) and the current line (included).
+            // Then remove all blank lines.
+            let trivia_range = TextRange::new(
+                state.last_non_comment_line_end,
+                self.locator.line_start(line.first_token_range.start()),
+            );
+            let trivia_text = self.locator.slice(trivia_range);
+            let mut trivia_without_blank_lines = trivia_text
+                .universal_newlines()
+                .filter_map(|line| (!line.trim_whitespace().is_empty()).then_some(line.as_str()))
+                .join(&self.stylist.line_ending());
+
+            let fix = if trivia_without_blank_lines.is_empty() {
+                Fix::safe_edit(Edit::range_deletion(trivia_range))
+            } else {
+                trivia_without_blank_lines.push_str(&self.stylist.line_ending());
+                Fix::safe_edit(Edit::range_replacement(
+                    trivia_without_blank_lines,
+                    trivia_range,
+                ))
+            };
+
+            diagnostic.set_fix(fix);
+
+            diagnostics.push(diagnostic);
+        }
+
+        if line.preceding_blank_lines < BLANK_LINES_TOP_LEVEL
+            && state
+                .previous_unindented_line_kind
+                .is_some_and(LogicalLineKind::is_class_function_or_decorator)
+            && line.indent_length == 0
+            && !line.is_comment_only
+            && !line.kind.is_class_function_or_decorator()
+            // Blank lines in stub files are used for grouping, don't enforce blank lines.
+            && !self.source_type.is_stub()
+        {
+            // E305
+            let mut diagnostic = Diagnostic::new(
+                BlankLinesAfterFunctionOrClass {
+                    actual_blank_lines: line.preceding_blank_lines.count(),
+                },
+                line.first_token_range,
+            );
+
+            if let Some(blank_lines_range) = line.blank_lines.range() {
+                diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+                    self.stylist
+                        .line_ending()
+                        .repeat(BLANK_LINES_TOP_LEVEL as usize),
+                    blank_lines_range,
+                )));
+            } else {
+                diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
+                    self.stylist
+                        .line_ending()
+                        .repeat(BLANK_LINES_TOP_LEVEL as usize),
+                    self.locator.line_start(line.first_token_range.start()),
+                )));
+            }
+
+            diagnostics.push(diagnostic);
+        }
+
+        if line.preceding_blank_lines == 0
+            // Only apply to nested functions.
+            && matches!(state.fn_status, Status::Inside(_))
+            && line.kind.is_class_function_or_decorator()
+            // Allow following a decorator (if there is an error it will be triggered on the first decorator).
+            && !matches!(state.follows, Follows::Decorator)
+            // The class's docstring can directly precede the first function.
+            && !matches!(state.follows, Follows::Docstring)
+            // Do not trigger when the def/class follows an "indenting token" (if/while/etc...).
+            && prev_indent_length.is_some_and(|prev_indent_length| prev_indent_length >= line.indent_length)
+            // Allow groups of one-liners.
+            && !(matches!(state.follows, Follows::Def) && line.last_token != TokenKind::Colon)
+            // Blank lines in stub files are only used for grouping. Don't enforce blank lines.
+            && !self.source_type.is_stub()
+        {
+            // E306
+            let mut diagnostic =
+                Diagnostic::new(BlankLinesBeforeNestedDefinition, line.first_token_range);
+
+            diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
+                self.stylist.line_ending().to_string(),
+                self.locator.line_start(line.first_token_range.start()),
+            )));
+
+            diagnostics.push(diagnostic);
+        }
+    }
+}
+
+#[derive(Clone, Debug, Default)]
+struct BlankLinesState {
     follows: Follows,
     fn_status: Status,
     class_status: Status,
@@ -615,262 +1002,6 @@ pub(crate) struct BlankLinesChecker {
     previous_unindented_line_kind: Option<LogicalLineKind>,
 }
 
-impl BlankLinesChecker {
-    /// E301, E302, E303, E304, E305, E306
-    pub(crate) fn check_lines(
-        &mut self,
-        tokens: &[LexResult],
-        locator: &Locator,
-        stylist: &Stylist,
-        indent_width: IndentWidth,
-        diagnostics: &mut Vec<Diagnostic>,
-    ) {
-        let mut prev_indent_length: Option<usize> = None;
-        let line_preprocessor = LinePreprocessor::new(tokens, locator, indent_width);
-
-        for logical_line in line_preprocessor {
-            self.check_line(
-                &logical_line,
-                prev_indent_length,
-                locator,
-                stylist,
-                diagnostics,
-            );
-            if !logical_line.is_comment_only {
-                prev_indent_length = Some(logical_line.indent_length);
-            }
-        }
-    }
-
-    #[allow(clippy::nonminimal_bool)]
-    fn check_line(
-        &mut self,
-        line: &LogicalLineInfo,
-        prev_indent_length: Option<usize>,
-        locator: &Locator,
-        stylist: &Stylist,
-        diagnostics: &mut Vec<Diagnostic>,
-    ) {
-        self.class_status.update(line);
-        self.fn_status.update(line);
-
-        // Don't expect blank lines before the first non comment line.
-        if self.is_not_first_logical_line {
-            if line.preceding_blank_lines == 0
-                // Only applies to methods.
-                && matches!(line.kind,  LogicalLineKind::Function)
-                && matches!(self.class_status, Status::Inside(_))
-                // The class/parent method's docstring can directly precede the def.
-                // Allow following a decorator (if there is an error it will be triggered on the first decorator).
-                && !matches!(self.follows, Follows::Docstring | Follows::Decorator)
-                // Do not trigger when the def follows an if/while/etc...
-                && prev_indent_length.is_some_and(|prev_indent_length| prev_indent_length >= line.indent_length)
-            {
-                // E301
-                let mut diagnostic =
-                    Diagnostic::new(BlankLineBetweenMethods, line.first_token_range);
-                diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
-                    stylist.line_ending().to_string(),
-                    locator.line_start(self.last_non_comment_line_end),
-                )));
-
-                diagnostics.push(diagnostic);
-            }
-
-            if line.preceding_blank_lines < BLANK_LINES_TOP_LEVEL
-                // Allow following a decorator (if there is an error it will be triggered on the first decorator).
-                && !matches!(self.follows, Follows::Decorator)
-                // Allow groups of one-liners.
-                && !(matches!(self.follows, Follows::Def) && !matches!(line.last_token, TokenKind::Colon))
-                // Only trigger on non-indented classes and functions (for example functions within an if are ignored)
-                && line.indent_length == 0
-                // Only apply to functions or classes.
-                && line.kind.is_top_level()
-            {
-                // E302
-                let mut diagnostic = Diagnostic::new(
-                    BlankLinesTopLevel {
-                        actual_blank_lines: line.preceding_blank_lines.count(),
-                    },
-                    line.first_token_range,
-                );
-
-                if let Some(blank_lines_range) = line.blank_lines.range() {
-                    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-                        stylist.line_ending().repeat(BLANK_LINES_TOP_LEVEL as usize),
-                        blank_lines_range,
-                    )));
-                } else {
-                    diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
-                        stylist.line_ending().repeat(BLANK_LINES_TOP_LEVEL as usize),
-                        locator.line_start(self.last_non_comment_line_end),
-                    )));
-                }
-
-                diagnostics.push(diagnostic);
-            }
-
-            let expected_blank_lines = if line.indent_length > 0 {
-                BLANK_LINES_METHOD_LEVEL
-            } else {
-                BLANK_LINES_TOP_LEVEL
-            };
-
-            if line.blank_lines > expected_blank_lines {
-                // E303
-                let mut diagnostic = Diagnostic::new(
-                    TooManyBlankLines {
-                        actual_blank_lines: line.blank_lines.count(),
-                    },
-                    line.first_token_range,
-                );
-
-                if let Some(blank_lines_range) = line.blank_lines.range() {
-                    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-                        stylist.line_ending().repeat(expected_blank_lines as usize),
-                        blank_lines_range,
-                    )));
-                }
-
-                diagnostics.push(diagnostic);
-            }
-
-            if matches!(self.follows, Follows::Decorator)
-                && !line.is_comment_only
-                && line.preceding_blank_lines > 0
-            {
-                // E304
-                let mut diagnostic = Diagnostic::new(
-                    BlankLineAfterDecorator {
-                        actual_blank_lines: line.preceding_blank_lines.count(),
-                    },
-                    line.first_token_range,
-                );
-
-                // Get all the lines between the last decorator line (included) and the current line (included).
-                // Then remove all blank lines.
-                let trivia_range = TextRange::new(
-                    self.last_non_comment_line_end,
-                    locator.line_start(line.first_token_range.start()),
-                );
-                let trivia_text = locator.slice(trivia_range);
-                let mut trivia_without_blank_lines = trivia_text
-                    .universal_newlines()
-                    .filter_map(|line| {
-                        (!line.trim_whitespace().is_empty()).then_some(line.as_str())
-                    })
-                    .join(&stylist.line_ending());
-
-                let fix = if trivia_without_blank_lines.is_empty() {
-                    Fix::safe_edit(Edit::range_deletion(trivia_range))
-                } else {
-                    trivia_without_blank_lines.push_str(&stylist.line_ending());
-                    Fix::safe_edit(Edit::range_replacement(
-                        trivia_without_blank_lines,
-                        trivia_range,
-                    ))
-                };
-
-                diagnostic.set_fix(fix);
-
-                diagnostics.push(diagnostic);
-            }
-
-            if line.preceding_blank_lines < BLANK_LINES_TOP_LEVEL
-                && self
-                    .previous_unindented_line_kind
-                    .is_some_and(LogicalLineKind::is_top_level)
-                && line.indent_length == 0
-                && !line.is_comment_only
-                && !line.kind.is_top_level()
-            {
-                // E305
-                let mut diagnostic = Diagnostic::new(
-                    BlankLinesAfterFunctionOrClass {
-                        actual_blank_lines: line.preceding_blank_lines.count(),
-                    },
-                    line.first_token_range,
-                );
-
-                if let Some(blank_lines_range) = line.blank_lines.range() {
-                    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-                        stylist.line_ending().repeat(BLANK_LINES_TOP_LEVEL as usize),
-                        blank_lines_range,
-                    )));
-                } else {
-                    diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
-                        stylist.line_ending().repeat(BLANK_LINES_TOP_LEVEL as usize),
-                        locator.line_start(line.first_token_range.start()),
-                    )));
-                }
-
-                diagnostics.push(diagnostic);
-            }
-
-            if line.preceding_blank_lines == 0
-                // Only apply to nested functions.
-                && matches!(self.fn_status, Status::Inside(_))
-                && line.kind.is_top_level()
-                // Allow following a decorator (if there is an error it will be triggered on the first decorator).
-                && !matches!(self.follows, Follows::Decorator)
-                // The class's docstring can directly precede the first function.
-                && !matches!(self.follows, Follows::Docstring)
-                // Do not trigger when the def/class follows an "indenting token" (if/while/etc...).
-                && prev_indent_length.is_some_and(|prev_indent_length| prev_indent_length >= line.indent_length)
-                // Allow groups of one-liners.
-                && !(matches!(self.follows, Follows::Def) && line.last_token != TokenKind::Colon)
-            {
-                // E306
-                let mut diagnostic =
-                    Diagnostic::new(BlankLinesBeforeNestedDefinition, line.first_token_range);
-
-                diagnostic.set_fix(Fix::safe_edit(Edit::insertion(
-                    stylist.line_ending().to_string(),
-                    locator.line_start(line.first_token_range.start()),
-                )));
-
-                diagnostics.push(diagnostic);
-            }
-        }
-
-        match line.kind {
-            LogicalLineKind::Class => {
-                if matches!(self.class_status, Status::Outside) {
-                    self.class_status = Status::Inside(line.indent_length);
-                }
-                self.follows = Follows::Other;
-            }
-            LogicalLineKind::Decorator => {
-                self.follows = Follows::Decorator;
-            }
-            LogicalLineKind::Function => {
-                if matches!(self.fn_status, Status::Outside) {
-                    self.fn_status = Status::Inside(line.indent_length);
-                }
-                self.follows = Follows::Def;
-            }
-            LogicalLineKind::Comment => {}
-            LogicalLineKind::Other => {
-                self.follows = Follows::Other;
-            }
-        }
-
-        if line.is_docstring {
-            self.follows = Follows::Docstring;
-        }
-
-        if !line.is_comment_only {
-            self.is_not_first_logical_line = true;
-
-            self.last_non_comment_line_end = line.logical_line_end;
-
-            if line.indent_length == 0 {
-                self.previous_unindented_line_kind = Some(line.kind);
-            }
-        }
-    }
-}
-
 #[derive(Copy, Clone, Debug)]
 enum LogicalLineKind {
     /// The clause header of a class definition
@@ -881,12 +1012,16 @@ enum LogicalLineKind {
     Function,
     /// A comment only line
     Comment,
+    /// An import statement
+    Import,
+    /// A from.. import statement
+    FromImport,
     /// Any other statement or clause header
     Other,
 }
 
 impl LogicalLineKind {
-    fn is_top_level(self) -> bool {
+    fn is_class_function_or_decorator(self) -> bool {
         matches!(
             self,
             LogicalLineKind::Class | LogicalLineKind::Function | LogicalLineKind::Decorator
