@@ -1,10 +1,11 @@
 use std::borrow::Cow;
 
 use anyhow::{Context, Result};
-use rustc_hash::FxHashMap;
+use rustc_hash::{FxHashMap, FxHashSet};
 
 use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
 use ruff_macros::{derive_message_formats, violation};
+use ruff_python_ast::helpers::any_over_expr;
 use ruff_python_ast::str::{leading_quote, trailing_quote};
 use ruff_python_ast::{self as ast, Expr, Keyword};
 use ruff_python_literal::format::{
@@ -15,7 +16,6 @@ use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
 
 use crate::checkers::ast::Checker;
-use crate::fix::edits::fits_or_shrinks;
 
 use crate::rules::pyflakes::format::FormatSummary;
 use crate::rules::pyupgrade::helpers::{curly_escape, curly_unescape};
@@ -56,7 +56,7 @@ impl Violation for FString {
 }
 
 /// Like [`FormatSummary`], but maps positional and keyword arguments to their
-/// values. For example, given `{a} {b}".format(a=1, b=2)`, `FormatFunction`
+/// values. For example, given `{a} {b}".format(a=1, b=2)`, [`FormatSummary`]
 /// would include `"a"` and `'b'` in `kwargs`, mapped to `1` and `2`
 /// respectively.
 #[derive(Debug)]
@@ -106,11 +106,11 @@ impl<'a> FormatSummaryValues<'a> {
         })
     }
 
-    /// Return the next positional argument.
-    fn arg_auto(&mut self) -> Option<&Expr> {
+    /// Return the next positional index.
+    fn arg_auto(&mut self) -> usize {
         let idx = self.auto_index;
         self.auto_index += 1;
-        self.arg_positional(idx)
+        idx
     }
 
     /// Return the positional argument at the given index.
@@ -146,9 +146,9 @@ fn parenthesize(expr: &Expr, text: &str, context: FormatContext) -> bool {
             Expr::BinOp(_)
             | Expr::UnaryOp(_)
             | Expr::BoolOp(_)
-            | Expr::NamedExpr(_)
+            | Expr::Named(_)
             | Expr::Compare(_)
-            | Expr::IfExp(_)
+            | Expr::If(_)
             | Expr::Lambda(_)
             | Expr::Await(_)
             | Expr::Yield(_)
@@ -166,7 +166,7 @@ fn parenthesize(expr: &Expr, text: &str, context: FormatContext) -> bool {
         // E.g., `{x, y}` should be parenthesized in `f"{(x, y)}"`.
         (
             _,
-            Expr::GeneratorExp(_)
+            Expr::Generator(_)
             | Expr::Dict(_)
             | Expr::Set(_)
             | Expr::SetComp(_)
@@ -175,7 +175,7 @@ fn parenthesize(expr: &Expr, text: &str, context: FormatContext) -> bool {
         (_, Expr::Subscript(ast::ExprSubscript { value, .. })) => {
             matches!(
                 value.as_ref(),
-                Expr::GeneratorExp(_)
+                Expr::Generator(_)
                     | Expr::Dict(_)
                     | Expr::Set(_)
                     | Expr::SetComp(_)
@@ -185,7 +185,7 @@ fn parenthesize(expr: &Expr, text: &str, context: FormatContext) -> bool {
         (_, Expr::Attribute(ast::ExprAttribute { value, .. })) => {
             matches!(
                 value.as_ref(),
-                Expr::GeneratorExp(_)
+                Expr::Generator(_)
                     | Expr::Dict(_)
                     | Expr::Set(_)
                     | Expr::SetComp(_)
@@ -195,7 +195,7 @@ fn parenthesize(expr: &Expr, text: &str, context: FormatContext) -> bool {
         (_, Expr::Call(ast::ExprCall { func, .. })) => {
             matches!(
                 func.as_ref(),
-                Expr::GeneratorExp(_)
+                Expr::Generator(_)
                     | Expr::Dict(_)
                     | Expr::Set(_)
                     | Expr::SetComp(_)
@@ -216,139 +216,182 @@ fn formatted_expr<'a>(expr: &Expr, context: FormatContext, locator: &Locator<'a>
     }
 }
 
-/// Convert a string `.format` call to an f-string.
-///
-/// Returns `None` if the string does not require conversion, and `Err` if the conversion
-/// is not possible.
-fn try_convert_to_f_string(
-    range: TextRange,
-    summary: &mut FormatSummaryValues,
-    locator: &Locator,
-) -> Result<Option<String>> {
-    let contents = locator.slice(range);
+#[derive(Debug, Clone)]
+enum FStringConversion {
+    /// The format string only contains literal parts.
+    Literal,
+    /// The format call uses arguments with side effects which are repeated within the
+    /// format string. For example: `"{x} {x}".format(x=foo())`.
+    SideEffects,
+    /// The format string should be converted to an f-string.
+    Convert(String),
+}
 
-    // Strip the unicode prefix. It's redundant in Python 3, and invalid when used
-    // with f-strings.
-    let contents = if contents.starts_with('U') || contents.starts_with('u') {
-        &contents[1..]
-    } else {
-        contents
-    };
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+enum IndexOrKeyword {
+    /// The field uses a positional index.
+    Index(usize),
+    /// The field uses a keyword name.
+    Keyword(String),
+}
 
-    // Temporarily strip the raw prefix, if present. It will be prepended to the result, before the
-    // 'f', to match the prefix order both the Ruff formatter (and Black) use when formatting code.
-    let raw = contents.starts_with('R') || contents.starts_with('r');
-    let contents = if raw { &contents[1..] } else { contents };
+impl FStringConversion {
+    /// Convert a string `.format` call to an f-string.
+    fn try_convert(
+        range: TextRange,
+        summary: &mut FormatSummaryValues,
+        locator: &Locator,
+    ) -> Result<Self> {
+        let contents = locator.slice(range);
 
-    // Remove the leading and trailing quotes.
-    let leading_quote = leading_quote(contents).context("Unable to identify leading quote")?;
-    let trailing_quote = trailing_quote(contents).context("Unable to identify trailing quote")?;
-    let contents = &contents[leading_quote.len()..contents.len() - trailing_quote.len()];
-    if contents.is_empty() {
-        return Ok(None);
-    }
+        // Strip the unicode prefix. It's redundant in Python 3, and invalid when used
+        // with f-strings.
+        let contents = if contents.starts_with('U') || contents.starts_with('u') {
+            &contents[1..]
+        } else {
+            contents
+        };
 
-    // Parse the format string.
-    let format_string = FormatString::from_str(contents)?;
+        // Temporarily strip the raw prefix, if present. It will be prepended to the result, before the
+        // 'f', to match the prefix order both the Ruff formatter (and Black) use when formatting code.
+        let raw = contents.starts_with('R') || contents.starts_with('r');
+        let contents = if raw { &contents[1..] } else { contents };
 
-    if format_string
-        .format_parts
-        .iter()
-        .all(|part| matches!(part, FormatPart::Literal(..)))
-    {
-        return Ok(None);
-    }
+        // Remove the leading and trailing quotes.
+        let leading_quote = leading_quote(contents).context("Unable to identify leading quote")?;
+        let trailing_quote =
+            trailing_quote(contents).context("Unable to identify trailing quote")?;
+        let contents = &contents[leading_quote.len()..contents.len() - trailing_quote.len()];
 
-    let mut converted = String::with_capacity(contents.len());
-    for part in format_string.format_parts {
-        match part {
-            FormatPart::Field {
-                field_name,
-                conversion_spec,
-                format_spec,
-            } => {
-                converted.push('{');
+        // If the format string is empty, it doesn't need to be converted.
+        if contents.is_empty() {
+            return Ok(Self::Literal);
+        }
 
-                let field = FieldName::parse(&field_name)?;
-                let arg = match field.field_type {
-                    FieldType::Auto => summary.arg_auto(),
-                    FieldType::Index(index) => summary.arg_positional(index),
-                    FieldType::Keyword(name) => summary.arg_keyword(&name),
-                }
-                .context("Unable to parse field")?;
-                converted.push_str(&formatted_expr(
-                    arg,
-                    if field.parts.is_empty() {
-                        FormatContext::Bare
-                    } else {
-                        FormatContext::Accessed
-                    },
-                    locator,
-                ));
+        // Parse the format string.
+        let format_string = FormatString::from_str(contents)?;
 
-                for part in field.parts {
-                    match part {
-                        FieldNamePart::Attribute(name) => {
-                            converted.push('.');
-                            converted.push_str(&name);
+        // If the format string contains only literal parts, it doesn't need to be converted.
+        if format_string
+            .format_parts
+            .iter()
+            .all(|part| matches!(part, FormatPart::Literal(..)))
+        {
+            return Ok(Self::Literal);
+        }
+
+        let mut converted = String::with_capacity(contents.len());
+        let mut seen = FxHashSet::default();
+        for part in format_string.format_parts {
+            match part {
+                FormatPart::Field {
+                    field_name,
+                    conversion_spec,
+                    format_spec,
+                } => {
+                    converted.push('{');
+
+                    let field = FieldName::parse(&field_name)?;
+
+                    // Map from field type to specifier.
+                    let specifier = match field.field_type {
+                        FieldType::Auto => IndexOrKeyword::Index(summary.arg_auto()),
+                        FieldType::Index(index) => IndexOrKeyword::Index(index),
+                        FieldType::Keyword(name) => IndexOrKeyword::Keyword(name),
+                    };
+
+                    let arg = match &specifier {
+                        IndexOrKeyword::Index(index) => {
+                            summary.arg_positional(*index).ok_or_else(|| {
+                                anyhow::anyhow!("Positional argument {index} is missing")
+                            })?
                         }
-                        FieldNamePart::Index(index) => {
-                            converted.push('[');
-                            converted.push_str(index.to_string().as_str());
-                            converted.push(']');
+                        IndexOrKeyword::Keyword(name) => {
+                            summary.arg_keyword(name).ok_or_else(|| {
+                                anyhow::anyhow!("Keyword argument '{name}' is missing")
+                            })?
                         }
-                        FieldNamePart::StringIndex(index) => {
-                            let quote = match trailing_quote {
-                                "'" | "'''" | "\"\"\"" => '"',
-                                "\"" => '\'',
-                                _ => unreachable!("invalid trailing quote"),
-                            };
-                            converted.push('[');
-                            converted.push(quote);
-                            converted.push_str(&index);
-                            converted.push(quote);
-                            converted.push(']');
+                    };
+
+                    // If the argument contains a side effect, and it's repeated in the format
+                    // string, we can't convert the format string to an f-string. For example,
+                    // converting `"{x} {x}".format(x=foo())` would result in `f"{foo()} {foo()}"`,
+                    // which would call `foo()` twice.
+                    if !seen.insert(specifier) {
+                        if any_over_expr(arg, &Expr::is_call_expr) {
+                            return Ok(Self::SideEffects);
                         }
                     }
-                }
 
-                if let Some(conversion_spec) = conversion_spec {
-                    converted.push('!');
-                    converted.push(conversion_spec);
-                }
+                    converted.push_str(&formatted_expr(
+                        arg,
+                        if field.parts.is_empty() {
+                            FormatContext::Bare
+                        } else {
+                            FormatContext::Accessed
+                        },
+                        locator,
+                    ));
 
-                if !format_spec.is_empty() {
-                    converted.push(':');
-                    converted.push_str(&format_spec);
-                }
+                    for part in field.parts {
+                        match part {
+                            FieldNamePart::Attribute(name) => {
+                                converted.push('.');
+                                converted.push_str(&name);
+                            }
+                            FieldNamePart::Index(index) => {
+                                converted.push('[');
+                                converted.push_str(index.to_string().as_str());
+                                converted.push(']');
+                            }
+                            FieldNamePart::StringIndex(index) => {
+                                let quote = match trailing_quote {
+                                    "'" | "'''" | "\"\"\"" => '"',
+                                    "\"" => '\'',
+                                    _ => unreachable!("invalid trailing quote"),
+                                };
+                                converted.push('[');
+                                converted.push(quote);
+                                converted.push_str(&index);
+                                converted.push(quote);
+                                converted.push(']');
+                            }
+                        }
+                    }
 
-                converted.push('}');
-            }
-            FormatPart::Literal(value) => {
-                converted.push_str(&curly_escape(&value));
+                    if let Some(conversion_spec) = conversion_spec {
+                        converted.push('!');
+                        converted.push(conversion_spec);
+                    }
+
+                    if !format_spec.is_empty() {
+                        converted.push(':');
+                        converted.push_str(&format_spec);
+                    }
+
+                    converted.push('}');
+                }
+                FormatPart::Literal(value) => {
+                    converted.push_str(&curly_escape(&value));
+                }
             }
         }
-    }
 
-    // Construct the format string.
-    let mut contents = String::with_capacity(usize::from(raw) + 1 + converted.len());
-    if raw {
-        contents.push('r');
+        // Construct the format string.
+        let mut contents = String::with_capacity(usize::from(raw) + 1 + converted.len());
+        if raw {
+            contents.push('r');
+        }
+        contents.push('f');
+        contents.push_str(leading_quote);
+        contents.push_str(&converted);
+        contents.push_str(trailing_quote);
+        Ok(Self::Convert(contents))
     }
-    contents.push('f');
-    contents.push_str(leading_quote);
-    contents.push_str(&converted);
-    contents.push_str(trailing_quote);
-    Ok(Some(contents))
 }
 
 /// UP032
-pub(crate) fn f_strings(
-    checker: &mut Checker,
-    call: &ast::ExprCall,
-    summary: &FormatSummary,
-    template: &Expr,
-) {
+pub(crate) fn f_strings(checker: &mut Checker, call: &ast::ExprCall, summary: &FormatSummary) {
     if summary.has_nested_parts {
         return;
     }
@@ -389,13 +432,16 @@ pub(crate) fn f_strings(
                 break range.start();
             }
             Some((Tok::String { .. }, range)) => {
-                match try_convert_to_f_string(range, &mut summary, checker.locator()) {
-                    Ok(Some(fstring)) => patches.push((range, fstring)),
+                match FStringConversion::try_convert(range, &mut summary, checker.locator()) {
+                    Ok(FStringConversion::Convert(fstring)) => patches.push((range, fstring)),
                     // Convert escaped curly brackets e.g. `{{` to `{` in literal string parts
-                    Ok(None) => patches.push((
+                    Ok(FStringConversion::Literal) => patches.push((
                         range,
                         curly_unescape(checker.locator().slice(range)).to_string(),
                     )),
+                    // If the format string contains side effects that would need to be repeated,
+                    // we can't convert it to an f-string.
+                    Ok(FStringConversion::SideEffects) => return,
                     // If any of the segments fail to convert, then we can't convert the entire
                     // expression.
                     Err(_) => return,
@@ -447,17 +493,6 @@ pub(crate) fn f_strings(
         contents.insert(0, ' ');
     }
 
-    // Avoid refactors that exceed the line length limit.
-    if !fits_or_shrinks(
-        &contents,
-        template.into(),
-        checker.locator(),
-        checker.settings.pycodestyle.max_line_length,
-        checker.settings.tab_size,
-    ) {
-        return;
-    }
-
     // Finally, avoid refactors that would introduce a runtime error.
     // For example, Django's `gettext` supports `format`-style arguments, but not f-strings.
     // See: https://docs.djangoproject.com/en/4.2/topics/i18n/translation
@@ -465,10 +500,10 @@ pub(crate) fn f_strings(
         expr.as_call_expr().is_some_and(|call| {
             checker
                 .semantic()
-                .resolve_call_path(call.func.as_ref())
-                .map_or(false, |call_path| {
+                .resolve_qualified_name(call.func.as_ref())
+                .map_or(false, |qualified_name| {
                     matches!(
-                        call_path.as_slice(),
+                        qualified_name.segments(),
                         ["django", "utils", "translation", "gettext" | "gettext_lazy"]
                     )
                 })
@@ -485,11 +520,12 @@ pub(crate) fn f_strings(
     //     0,  # 0
     // )
     // ```
-    if !checker
+    let has_comments = checker
         .indexer()
         .comment_ranges()
-        .intersects(call.arguments.range())
-    {
+        .intersects(call.arguments.range());
+
+    if !has_comments {
         diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
             contents,
             call.range(),
