@@ -450,6 +450,7 @@ pub(crate) fn find_parameter_separators(
     // * `f(a, /, b)`
     // * `f(a, /, *b)`
     // * `f(a, /, *, b)`
+    // * `f(a, /, *, **b)`
     // * `f(a, /)`
     let slash_following_start = parameters
         .args
@@ -457,6 +458,7 @@ pub(crate) fn find_parameter_separators(
         .map(Ranged::start)
         .or(parameters.vararg.as_ref().map(|first| first.start()))
         .or(star.as_ref().map(|star| star.separator.start()))
+        .or(parameters.kwarg.as_deref().map(Ranged::start))
         .unwrap_or(parameters.end());
     let slash = slash.map(|(preceding_end, slash)| ParameterSeparator {
         preceding_end,
